<?php
/**
 * ANTON
 *
 * REST API functions : CRUD
 *
 */


/**
 * Authenticate and get the session id back for later use
 * 
 * 
 * 
 * @param unknown $endpointUrl
 * @param unknown $userName
 * @param unknown $userAccessKey : you find that in the personal settings of the ANTON user you want to log in as
 * @return string|unknown[]
 */
function vtiger_webservice_login($endpointUrl,$userName,$userAccessKey)
{
	/**
	 * get ChallengeToken
	 */
	
	
	$requesturi = "{$endpointUrl}?operation=getchallenge&username={$userName}";

	//getchallenge request must be a GET request.
	$request = new HTTP_Request2($requesturi,HTTP_Request2::METHOD_GET);
	

	try {
	    $response = $request->send();
	    if (200 == $response->getStatus()) {
	        //echo $response->getBody().PHP_EOL;
	        $jsonResponse = Zend_JSON::decode($response->getBody());
	        
	        //check for whether the requested operation was successful or not.
	        if($jsonResponse['success']==false) {
	            //handle the failure case.
	            if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "GetChallenge failed";
	            return $jsonResponse;
	        }
	        
	        //operation was successful get the token from the reponse.
	        $challengeToken = $jsonResponse['result']['token'];
	    } else {
	        echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
	    }
	} catch (HTTP_Request2_Exception $e) {
	    echo 'Error: ' . $e->getMessage().PHP_EOL;
	}
	
	unset($request);
	

	/**
	 * Login and get SessionID
	 */
	
	//create md5 string concatenating user accesskey from my preference page
	//and the challenge token obtained from get challenge result.
	$generatedKey = md5($challengeToken.$userAccessKey);
	
	//login request must be POST request.
	
	$request = new HTTP_Request2($endpointUrl,HTTP_Request2::METHOD_POST);	
	$request->addPostParameter('operation', 'login');
	$request->addPostParameter('username', $userName);
	$request->addPostParameter('accessKey', $generatedKey);
	
	try {
	    $response = $request->send();
	    if (200 == $response->getStatus()) {
	        //echo $response->getBody().PHP_EOL;
	        $jsonResponse = Zend_JSON::decode($response->getBody());
	        
	        //check for whether the requested operation was successful or not.
	        if($jsonResponse['success']==false) {
	            //handle the failure case.
	            if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "Login failed";
	            return $jsonResponse;
	        }
	        
	        //login successful extract sessionId and userId from LoginResult to it can used for further calls.
	        $sessionId = $jsonResponse['result']['sessionName'];
	        $userId = $jsonResponse['result']['userId'];
	        
	    } else {
	        echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
	    }
	} catch (HTTP_Request2_Exception $e) {
	    echo 'Error: ' . $e->getMessage().PHP_EOL;
	}
	
	unset($request);
	
	
	$ret = array();
	$ret['sessionId'] = $sessionId;
	$ret['userId'] = $userId;
	
	return $ret;
}




/**
 * Update a CRM record, previously retrieved
 * 
 * @param unknown $changedObject
 * @return string|mixed|array|StdClass|NULL
 */
function update_crm_record($changedObject)
{
	global $endpointUrl, $sessionId, $userId;
	
	
	//encode the object in JSON format to communicate with the server.
	$objectJson = Zend_JSON::encode($changedObject);
	
	// json encoding weakness
	$objectJson = str_replace(array('<', '>', '&'), array('\u003c', '\u003e', '\u0026'), $objectJson);

	
	$request = new HTTP_Request2($endpointUrl,HTTP_Request2::METHOD_POST);
	$request->addPostParameter('sessionName', $sessionId);
	$request->addPostParameter('operation', 'update');
	$request->addPostParameter('element', $objectJson);
	
	try {
	    $response = $request->send();
	    if (200 == $response->getStatus()) {
	        //echo $response->getBody().PHP_EOL;
	        $jsonResponse = Zend_JSON::decode($response->getBody());
	        
	        //check for whether the requested operation was successful or not.
	        if($jsonResponse['success']==false) {
	            //handle the failure case.
	            if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "Update failed";
	            return $jsonResponse;
	        }
	        
	        $updatedObject = $jsonResponse['result'];
	        
	    } else {
	        echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
	    }
	} catch (HTTP_Request2_Exception $e) {
	    echo 'Error: ' . $e->getMessage().PHP_EOL;
	}
	
	unset($request);
	
	unset($response);
	unset($jsonResponse);
	
	return $updatedObject;
	
	
}



/**
 * Create a CRM record 
 * 
 * @param string $module
 * @param unknown $data
 * @return string|unknown
 */
function create_record($module='Accounts',$data)
{
	global $endpointUrl, $sessionId, $userId;
	
	
	//encode the object in JSON format to communicate with the server.
	$objectJson = Zend_JSON::encode($data);
	
	// json encoding weakness
	$objectJson = str_replace(array('<', '>', '&'), array('\u003c', '\u003e', '\u0026'), $objectJson);
	
	//name of the module for which the entry has to be created.
	$moduleName = $module;
	
	
	$request = new HTTP_Request2($endpointUrl,HTTP_Request2::METHOD_POST);
	$request->addPostParameter('sessionName', $sessionId);
	$request->addPostParameter('operation', 'create');
	$request->addPostParameter('element', $objectJson);
	$request->addPostParameter('elementType', $moduleName);
	
	try {
	    $response = $request->send();
	    if (200 == $response->getStatus()) {
	        //echo $response->getBody().PHP_EOL;
	        $jsonResponse = Zend_JSON::decode($response->getBody());
	        
	        //check for whether the requested operation was successful or not.
	        if($jsonResponse['success']==false) {
	            //handle the failure case.
	            if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "Login failed";
	            return $jsonResponse;
	        }
	        
	        $savedObject = $jsonResponse['result'];
	        $id = $savedObject['id'];
	        
	    } else {
	        echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
	    }
	} catch (HTTP_Request2_Exception $e) {
	    echo 'Error: ' . $e->getMessage().PHP_EOL;
	}
	
	unset($request);
	
	unset($response);
	unset($jsonResponse);
	
	return $id;

}

/**
 * Retrieve a CRM record by ID 
 * 
 * @param unknown $wsid
 * @return unknown|string|mixed|array|StdClass|NULL
 */
function retrieve_object($wsid)
{
	global $endpointUrl, $sessionId, $userId;
	global $adb, $current_user;
	
	
	// id => webservice format "TYPExRECORD" : type = 10 (look into vtiger_ws_entity)
	$uri = "{$endpointUrl}?sessionName={$sessionId}&operation=retrieve&id={$wsid}";
	return(send_query($uri));
	
}

/**
 * Delete CRM Record by ID
 * 
 * @param unknown $wsid
 * @return unknown|string|mixed|array|StdClass|NULL
 */
function delete_object($wsid)
{
    global $endpointUrl, $sessionId, $userId;
    global $adb, $current_user;
    
    
    // id => webservice format "TYPExRECORD" : type = 10 (look into vtiger_ws_entity)
    $uri = "{$endpointUrl}?sessionName={$sessionId}&operation=delete&id={$wsid}";
    return(send_query($uri));
    
}



/**
 * Send query
 * reduce the overall code ;-)
 * @param unknown_type $params
 */
function send_query($requesturi)
{
	global $endpointUrl, $sessionId, $userId;
	

	$ret = array();
	
	// request must be a GET request.
	$request = new HTTP_Request2($requesturi,HTTP_Request2::METHOD_GET);
	
	try {
	    $response = $request->send();
	    if (200 == $response->getStatus()) {
	        //echo $response->getBody().PHP_EOL;
	        $jsonResponse = Zend_JSON::decode($response->getBody());
	        
	        //check for whether the requested operation was successful or not.
	        if($jsonResponse['success']==false) {
	            //handle the failure case.
	            if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "Request failed";
	            return $jsonResponse;
	        }
	        
	        //Array of vtigerObjects
	        $retrievedObjects = $jsonResponse['result'];
	        
	        if(count($retrievedObjects)<1) {
	            echo "Keine Treffer ...".PHP_EOL;
	        }
	        if(count($retrievedObjects)>=1) {
	            echo count($retrievedObjects)." Treffer ...".PHP_EOL;
	        }
	        	        
	    } else {
	        echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
	    }
	} catch (HTTP_Request2_Exception $e) {
	    echo 'Error: ' . $e->getMessage().PHP_EOL;
	}
	
	
	unset($request);
	
	return 	$retrievedObjects;
	
}

/**
 * Retrieve record by sql like query 
 *  
 * @param unknown $endpointUrl
 * @param unknown $sessionId
 * @param string $sql
 * @return string|mixed|NULL|array|StdClass|mixed[]|array[]|StdClass[]|NULL[]|boolean|number|unknown|boolean|array|string
 */
function query_record($endpointUrl,$sessionId,$sql='')
{

    $ret = array();
    
    // we must feed the request with an simple SQL-like statement
    // Example : select * from Contacts where email='antontest@e-wert.de';
    
    $requesturi = "{$endpointUrl}?sessionName={$sessionId}&operation=query&query={$sql}";
    
    // request must be a GET request.
    $request = new HTTP_Request2($requesturi,HTTP_Request2::METHOD_GET);
    
    try {
        $response = $request->send();
        if (200 == $response->getStatus()) {
            //echo $response->getBody().PHP_EOL;
            $jsonResponse = Zend_JSON::decode($response->getBody());
            
            //check for whether the requested operation was successful or not.
            if($jsonResponse['success']==false) {
                //handle the failure case.
                if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "ListTypes failed";
                return $jsonResponse;
            }
            
            //Array of vtigerObjects
            $retrievedObjects = $jsonResponse['result'];
            
            if(count($retrievedObjects)!=1) {
                echo "Keine oder zu viele Treffer ...".PHP_EOL;
                
                unset($request);
                
                return $ret;
            }
            
            $ret = $retrievedObjects[0];
                        
        } else {
            echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
        }
    } catch (HTTP_Request2_Exception $e) {
        echo 'Error: ' . $e->getMessage().PHP_EOL;
    }
    
    unset($request);
    
    return $ret;
 
}




/**
 * Retrieve and print info about objects
 * 
 * You'll get all fields including custromfiels with their labels and internal field names
 * With that help you can "read" the output array from i.e. retrieve_record above
 *  
 * @param unknown $endpointUrl
 * @param unknown $sessionId
 */
function print_vtiger_info($endpointUrl,$sessionId)
{

    $requesturi = "{$endpointUrl}?sessionName={$sessionId}&operation=listtypes";
    
    // request must be a GET request.
    $request = new HTTP_Request2($requesturi,HTTP_Request2::METHOD_GET);
    
    try {
        $response = $request->send();
        if (200 == $response->getStatus()) {
            //echo $response->getBody().PHP_EOL;
            $jsonResponse = Zend_JSON::decode($response->getBody());
            
            //check for whether the requested operation was successful or not.
            if($jsonResponse['success']==false) {
                //handle the failure case.
                if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "ListTypes failed";
                return $jsonResponse;
            }
            
            //Get the List of all the modules accessible.
            $modules = $jsonResponse['result']['types'];
            echo "Modules: ".print_r($modules,true).PHP_EOL;
            
            
        } else {
            echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
        }
    } catch (HTTP_Request2_Exception $e) {
        echo 'Error: ' . $e->getMessage().PHP_EOL;
    }
    
    unset($request);
    
    
    /**
     * List fields of given module
     */
    
    //vtiger Object name which need be described or whose information is requested.
    $moduleName = 'Contacts';
    
    //use sessionId created at the time of login.
    $requesturi = "{$endpointUrl}?sessionName={$sessionId}&operation=describe&elementType={$moduleName}";
    
    // request must be a GET request.
    $request = new HTTP_Request2($requesturi,HTTP_Request2::METHOD_GET);
    
    try {
        $response = $request->send();
        if (200 == $response->getStatus()) {
            //echo $response->getBody().PHP_EOL;
            $jsonResponse = Zend_JSON::decode($response->getBody());
            
            //check for whether the requested operation was successful or not.
            if($jsonResponse['success']==false) {
                //handle the failure case.
                if(!isset($jsonResponse['error']['message']) || empty($jsonResponse['error']['message'])) $jsonResponse['error']['message'] = "Describe failed";
                return $jsonResponse;
            }
            
            //get describe result object.
            $description = $jsonResponse['result'];
            echo "{$moduleName}: ".print_r($description,true).PHP_EOL;
            
            
        } else {
            echo 'Unexpected HTTP status: ' . $response->getStatus() . ' ' . $response->getReasonPhrase().PHP_EOL;
        }
    } catch (HTTP_Request2_Exception $e) {
        echo 'Error: ' . $e->getMessage().PHP_EOL;
    }
    
    unset($request);
    
  
}





/**
 * Helper function : hexdump
 * @param unknown $data
 * @param string $newline
 */
function hexdump($data, $newline="\n")
{
	static $from = '';
	static $to = '';
	
	static $width = 16; # number of bytes per line
	
	static $pad = '.'; # padding for non-visible characters
	
	if ($from==='')
	{
		for ($i=0; $i<=0xFF; $i++)
		{
			$from .= chr($i);
			$to .= ($i >= 0x20 && $i <= 0x7E) ? chr($i) : $pad;
		}
	}
	
	$hex = str_split(bin2hex($data), $width*2);
	$chars = str_split(strtr($data, $from, $to), $width);
	
	$offset = 0;
	foreach ($hex as $i => $line)
	{
		echo sprintf('%6X',$offset).' : '.implode(' ', str_split($line,2)) . ' [' . $chars[$i] . ']' . $newline;
		$offset += $width;
	}
}




?>